using System;
using System.IO;
using Server;
using System.Text;
using System.Collections;
using System.Net;
using Server.Accounting;
using Server.Mobiles;
using Server.Items;
using Server.Menus;
using Server.Menus.Questions;
using Server.Menus.ItemLists;
using Server.Network;
using Server.Spells;
using Server.Targeting;
using Server.Targets;
using Server.Gumps;

namespace Server.Scripts.Commands
{
	public class CommandHandlers
	{
		public static void Initialize()
		{
			Server.Commands.CommandPrefix = "[";

			Properties.Register();

			Register( "Go", AccessLevel.Counselor, new CommandEventHandler( Go_OnCommand ) );

			Register( "Delete", AccessLevel.GameMaster, new CommandEventHandler( Delete_OnCommand ) );
			Register( "Remove", AccessLevel.GameMaster, new CommandEventHandler( Delete_OnCommand ) );

			Register( "ShaveHair", AccessLevel.GameMaster, new CommandEventHandler( ShaveHair_OnCommand ) );
			Register( "ShaveBeard", AccessLevel.GameMaster, new CommandEventHandler( ShaveBeard_OnCommand ) );

			Register( "Kill", AccessLevel.GameMaster, new CommandEventHandler( Kill_OnCommand ) );

			Register( "Resurrect", AccessLevel.GameMaster, new CommandEventHandler( Resurrect_OnCommand ) );
			Register( "Res", AccessLevel.GameMaster, new CommandEventHandler( Resurrect_OnCommand ) );

			Register( "NewXYZ", AccessLevel.GameMaster, new CommandEventHandler( NewXYZ_OnCommand ) );
			Register( "NewX", AccessLevel.GameMaster, new CommandEventHandler( NewX_OnCommand ) );
			Register( "NewY", AccessLevel.GameMaster, new CommandEventHandler( NewY_OnCommand ) );
			Register( "NewZ", AccessLevel.GameMaster, new CommandEventHandler( NewZ_OnCommand ) );

			Register( "IncXYZ", AccessLevel.GameMaster, new CommandEventHandler( IncXYZ_OnCommand ) );
			Register( "IncX", AccessLevel.GameMaster, new CommandEventHandler( IncX_OnCommand ) );
			Register( "IncY", AccessLevel.GameMaster, new CommandEventHandler( IncY_OnCommand ) );
			Register( "IncZ", AccessLevel.GameMaster, new CommandEventHandler( IncZ_OnCommand ) );

			Register( "AutoPageNotify", AccessLevel.Counselor, new CommandEventHandler( APN_OnCommand ) );
			Register( "APN", AccessLevel.Counselor, new CommandEventHandler( APN_OnCommand ) );

			Register( "Animate", AccessLevel.Counselor, new CommandEventHandler( Animate_OnCommand ) );

			Register( "Tele", AccessLevel.Counselor, new CommandEventHandler( Teleport_OnCommand ) );
			Register( "Teleport", AccessLevel.Counselor, new CommandEventHandler( Teleport_OnCommand ) );

			Register( "Hide", AccessLevel.Counselor, new CommandEventHandler( Hide_OnCommand ) );
			Register( "Unhide", AccessLevel.Counselor, new CommandEventHandler( Unhide_OnCommand ) );

			Register( "Cast", AccessLevel.Counselor, new CommandEventHandler( Cast_OnCommand ) );

			Register( "Stuck", AccessLevel.Counselor, new CommandEventHandler( Stuck_OnCommand ) );

			Register( "Invul", AccessLevel.GameMaster, new CommandEventHandler( Immortal_OnCommand ) );
			Register( "Immortal", AccessLevel.GameMaster, new CommandEventHandler( Immortal_OnCommand ) );

			Register( "NoInvul", AccessLevel.GameMaster, new CommandEventHandler( Mortal_OnCommand ) );
			Register( "Mortal", AccessLevel.GameMaster, new CommandEventHandler( Mortal_OnCommand ) );

			Register( "Help", AccessLevel.Player, new CommandEventHandler( Help_OnCommand ) );

			Register( "Save", AccessLevel.Administrator, new CommandEventHandler( Save_OnCommand ) );

			Register( "Kick", AccessLevel.GameMaster, new CommandEventHandler( Kick_OnCommand ) );
			Register( "Ban", AccessLevel.Administrator, new CommandEventHandler( Ban_OnCommand ) );

			Register( "Firewall", AccessLevel.Administrator, new CommandEventHandler( Firewall_OnCommand ) );

			Register( "Move", AccessLevel.GameMaster, new CommandEventHandler( Move_OnCommand ) );
			Register( "Client", AccessLevel.Counselor, new CommandEventHandler( Client_OnCommand ) );

			Register( "SMsg", AccessLevel.Counselor, new CommandEventHandler( StaffMessage_OnCommand ) );
			Register( "SM", AccessLevel.Counselor, new CommandEventHandler( StaffMessage_OnCommand ) );
			Register( "S", AccessLevel.Counselor, new CommandEventHandler( StaffMessage_OnCommand ) );
       
			Register( "BCast", AccessLevel.Counselor, new CommandEventHandler( BroadcastMessage_OnCommand ) );
			Register( "BC", AccessLevel.Counselor, new CommandEventHandler( BroadcastMessage_OnCommand ) );
			Register( "B", AccessLevel.Counselor, new CommandEventHandler( BroadcastMessage_OnCommand ) );

			Register( "Dismount", AccessLevel.GameMaster, new CommandEventHandler( Dismount_OnCommand ) );

			Register( "Bank", AccessLevel.GameMaster, new CommandEventHandler( Bank_OnCommand ) );

			Register( "Echo", AccessLevel.Player, new CommandEventHandler( Echo_OnCommand ) );

			Register( "Sound", AccessLevel.GameMaster, new CommandEventHandler( Sound_OnCommand ) );
			Register( "PrivSound", AccessLevel.GameMaster, new CommandEventHandler( PrivSound_OnCommand ) );

			Register( "ViewEquip", AccessLevel.GameMaster, new CommandEventHandler( ViewEquip_OnCommand ) );

			Register( "DumpTimers", AccessLevel.Administrator, new CommandEventHandler( DumpTimers_OnCommand ) );

			Register( "Light", AccessLevel.Counselor, new CommandEventHandler( Light_OnCommand ) );
			Register( "Stats", AccessLevel.Counselor, new CommandEventHandler( Stats_OnCommand ) );
		}

		public static void Register( string command, AccessLevel access, CommandEventHandler handler )
		{
			Server.Commands.Register( command, access, handler );
		}

		private class ViewEqTarget : Target
		{
			public ViewEqTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
					from.SendMenu( new EquipMenu( from, (Mobile)targeted, GetEquip( (Mobile)targeted ) ) );
			}

			private static ItemListEntry[] GetEquip( Mobile m )
			{
				ItemListEntry[] entries = new ItemListEntry[m.Items.Count];

				for ( int i = 0; i < m.Items.Count; ++i )
				{
					Item item = (Item)m.Items[i];

					entries[i] = new ItemListEntry( String.Format( "{0}: {1}", item.Layer, item.GetType().Name ), item.ItemID, item.Hue );
				}

				return entries;
			}

			private class EquipMenu : ItemListMenu
			{
				private Mobile m_Mobile;

				public EquipMenu( Mobile from, Mobile m, ItemListEntry[] entries ) : base( "Equipment", entries )
				{
					m_Mobile = m;

					CommandLogging.WriteLine( from, "{0} {1} getting equip for {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( m ) );
				}

				public override void OnResponse( NetState state, int index )
				{
					if ( index >= 0 && index < m_Mobile.Items.Count )
					{
						Item item = (Item)m_Mobile.Items[index];

						state.Mobile.SendMenu( new EquipDetailsMenu( m_Mobile, item ) );
					}
				}

				private class EquipDetailsMenu : QuestionMenu
				{
					private Mobile m_Mobile;
					private Item m_Item;

					public EquipDetailsMenu( Mobile m, Item item ) : base( String.Format( "{0}: {1}", item.Layer, item.GetType().Name ), new string[]{"Move","Delete","Props"})
					{
						m_Mobile = m;
						m_Item = item;
					}

					public override void OnCancel( NetState state )
					{
						state.Mobile.SendMenu( new EquipMenu( state.Mobile, m_Mobile, ViewEqTarget.GetEquip( m_Mobile ) ) );
					}

					public override void OnResponse( NetState state, int index )
					{
						if ( index == 0 )
						{
							CommandLogging.WriteLine( state.Mobile, "{0} {1} moving equip item {2} of {3}", state.Mobile.AccessLevel, CommandLogging.Format( state.Mobile ), CommandLogging.Format( m_Item ), CommandLogging.Format( m_Mobile ) );
							state.Mobile.Target = new MoveTarget( m_Item );
						}
						else if ( index == 1 )
						{
							CommandLogging.WriteLine( state.Mobile, "{0} {1} deleting equip item {2} of {3}", state.Mobile.AccessLevel, CommandLogging.Format( state.Mobile ), CommandLogging.Format( m_Item ), CommandLogging.Format( m_Mobile ) );
							m_Item.Delete();
						}
						else if ( index == 2 )
						{
							CommandLogging.WriteLine( state.Mobile, "{0} {1} opening props for equip item {2} of {3}", state.Mobile.AccessLevel, CommandLogging.Format( state.Mobile ), CommandLogging.Format( m_Item ), CommandLogging.Format( m_Mobile ) );
							state.Mobile.SendGump( new PropertiesGump( state.Mobile, m_Item ) );
						}
					}
				}
			}
		}

		[Usage( "DumpTimers" )]
		[Description( "Generates a log file of all currently executing timers. Used for tracing timer leaks." )]
		public static void DumpTimers_OnCommand( CommandEventArgs e )
		{
			try
			{
				using ( StreamWriter sw = new StreamWriter( "timerdump.log", true ) )
					Timer.DumpInfo( sw );
			}
			catch
			{
			}
		}

		[Usage( "ViewEquip" )]
		[Description( "Lists equipment of a targeted mobile. From the list you can move, delete, or open props." )]
		public static void ViewEquip_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new ViewEqTarget();
		}

		private class PrivSoundTarget : Target
		{
			private int m_Index;

			public PrivSoundTarget( int index ) : base( -1, false, TargetFlags.None )
			{
				m_Index = index;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					CommandLogging.WriteLine( from, "{0} {1} playing sound {2} for {3}", from.AccessLevel, CommandLogging.Format( from ), m_Index, CommandLogging.Format( targeted ) );
					((Mobile)targeted).Send( new PlaySound( m_Index, ((Mobile)targeted).Location ) );
				}
			}
		}

		[Usage( "PrivSound <index>" )]
		[Description( "Plays a sound to a given target." )]
		public static void PrivSound_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
				e.Mobile.Target = new PrivSoundTarget( e.GetInt32( 0 ) );
			else
				e.Mobile.SendMessage( "Format: PrivSound <index>" );
		}

		[Usage( "Sound <index> [toAll=true]" )]
		[Description( "Plays a sound to players within 12 tiles of you. The (toAll) argument specifies to everyone, or just those who can see you." )]
		public static void Sound_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
				PlaySound( e.Mobile, e.GetInt32( 0 ), true );
			else if ( e.Length == 2 )
				PlaySound( e.Mobile, e.GetInt32( 0 ), e.GetBoolean( 1 ) );
			else
				e.Mobile.SendMessage( "Format: Sound <index> [toAll]" );
		}

		private static void PlaySound( Mobile m, int index, bool toAll )
		{
			Map map = m.Map;

			if ( map == null )
				return;

			CommandLogging.WriteLine( m, "{0} {1} playing sound {2} (toAll={3})", m.AccessLevel, CommandLogging.Format( m ), index, toAll );

			Packet p = new PlaySound( index, m.Location );

			IPooledEnumerable eable = map.GetClientsInRange( m.Location, 12 );

			foreach ( NetState state in eable )
				if ( toAll || state.Mobile.CanSee( m ) )
					state.Send( p );

			eable.Free();
		}

		private class BankTarget : Target
		{
			public BankTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					BankBox box = ((Mobile)targeted).BankBox;

					if ( box != null )
					{
						CommandLogging.WriteLine( from, "{0} {1} opening bank box of {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );
						box.DisplayTo( from );
					}
				}
			}
		}

		[Usage( "Echo <text>" )]
		[Description( "Relays (text) as a system message." )]
		public static void Echo_OnCommand( CommandEventArgs e )
		{
			string toEcho = e.ArgString.Trim();

			if ( toEcho.Length > 0 )
				e.Mobile.SendMessage( toEcho );
			else
				e.Mobile.SendMessage( "Format: Echo \"<text>\"" );
		}

		[Usage( "Bank" )]
		[Description( "Opens the bank box of a given target." )]
		public static void Bank_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new BankTarget();
		}

		private class DismountTarget : Target
		{
			public DismountTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					CommandLogging.WriteLine( from, "{0} {1} dismounting {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );

					Mobile targ = (Mobile)targeted;

					for ( int i = 0; i < targ.Items.Count; ++i )
					{
						Item item = (Item)targ.Items[i];

						if ( item is IMountItem )
						{
							IMount mount = ((IMountItem)item).Mount;

							if ( mount != null )
								mount.Rider = null;

							if ( targ.Items.IndexOf( item ) == -1 )
								--i;
						}
					}

					for ( int i = 0; i < targ.Items.Count; ++i )
					{
						Item item = (Item)targ.Items[i];

						if ( item.Layer == Layer.Mount )
						{
							item.Delete();
							--i;
						}
					}
				}
			}
		}

		[Usage( "Dismount" )]
		[Description( "Forcefully dismounts a given target." )]
		public static void Dismount_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DismountTarget();
		}

		private class ClientTarget : Target
		{
			public ClientTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile targ = (Mobile)targeted;

					if ( targ.NetState != null )
					{
						CommandLogging.WriteLine( from, "{0} {1} opening client menu of {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );
						from.SendGump( new ClientGump( from, targ.NetState ) );
					}
				}
			}
		}

		[Usage( "Client" )]
		[Description( "Opens the client gump menu for a given player." )]
		private static void Client_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new ClientTarget();
		}

		[Usage( "Move" )]
		[Description( "Repositions a targeted item or mobile." )]
		private static void Move_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new PickMoveTarget();
		}

		private class FirewallTarget : Target
		{
			public FirewallTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile targ = (Mobile)targeted;

					NetState state = targ.NetState;

					if ( state != null )
					{
						CommandLogging.WriteLine( from, "{0} {1} firewalling {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );

						try
						{
							Firewall.Add( ((IPEndPoint)state.Socket.RemoteEndPoint).Address );
						}
						catch
						{
						}
					}
				}
			}
		}

		[Usage( "Firewall" )]
		[Description( "Adds a targeted player to the firewall (list of blocked IPs). This does not ban or kick." )]
		private static void Firewall_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new FirewallTarget();
		}

		private class KickBanTarget : Target
		{
			private bool m_Ban;

			public KickBanTarget( bool ban ) : base( -1, false, TargetFlags.None )
			{
				m_Ban = ban;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile targ = (Mobile)targeted;

					if ( from.AccessLevel > targ.AccessLevel )
					{
						NetState fromState = from.NetState, targState = targ.NetState;

						if ( fromState != null && targState != null )
						{
							Account fromAccount = fromState.Account as Account;
							Account targAccount = targState.Account as Account;

							if ( fromAccount != null && targAccount != null )
							{
								CommandLogging.WriteLine( from, "{0} {1} {2} {3}", from.AccessLevel, CommandLogging.Format( from ), m_Ban ? "banning" : "kicking", CommandLogging.Format( targeted ) );

								targ.Say( "I've been {0}!", m_Ban ? "banned" : "kicked" );

								targState.Dispose();

								if ( m_Ban )
									targAccount.Banned = true;
							}
						}
					}
				}
			}
		}

		[Usage( "Kick" )]
		[Description( "Disconnects a targeted player." )]
		private static void Kick_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KickBanTarget( false );
		}

		[Usage( "Ban" )]
		[Description( "Bans the account of a targeted player." )]
		private static void Ban_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KickBanTarget( true );
		}

		[Usage( "Save" )]
		[Description( "Saves the world." )]
		private static void Save_OnCommand( CommandEventArgs e )
		{
			World.Save();
		}

		[Usage( "Go [name | (<x> <y> [z])]" )]
		[Description( "With no arguments, this command brings up the go menu. With one argument, (name), you are moved to that regions \"go location.\" Two or three arguments, (x y [z]), will move your character to that location." )]
		private static void Go_OnCommand( CommandEventArgs e )
		{
			Mobile from = e.Mobile;

			if ( e.Length == 0 )
			{
				GoGump.DisplayTo( from );
			}
			else if ( e.Length == 1 )
			{
				try
				{
					string name = e.GetString( 0 ).ToLower();

					ArrayList list = Region.Table[from.Map.MapID];

					for ( int i = 0; i < list.Count; ++i )
					{
						Region r = (Region)list[i];

						if ( r.Name.ToLower() == name )
						{
							from.Location = new Point3D( r.GoLocation );
							return;
						}
					}
				}
				catch
				{
				}

				from.SendMessage( "Region name not found" );
			}
			else if ( e.Length == 2 )
			{
				Map map = from.Map;

				if ( map != null )
				{
					int x = e.GetInt32( 0 ), y = e.GetInt32( 1 );
					int z = map.GetAverageZ( x, y );

					from.Location = new Point3D( x, y, z );
				}
			}
			else if ( e.Length == 3 )
			{
				from.Location = new Point3D( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 2 ) );
			}
			else
			{
				from.SendMessage( "Format: Go [name | <x y [z]>]" );
			}
		}

		[Usage( "Help" )]
		[Description( "Lists all available commands." )]
		public static void Help_OnCommand( CommandEventArgs e )
		{
			Mobile m = e.Mobile;

			ArrayList list = new ArrayList();

			foreach ( CommandEntry entry in Server.Commands.Entries.Values )
			{
				if ( m.AccessLevel >= entry.AccessLevel )
					list.Add( entry );
			}

			list.Sort();

			StringBuilder sb = new StringBuilder();

			if ( list.Count > 0 )
				sb.Append( ((CommandEntry)list[0]).Command );

			for ( int i = 1; i < list.Count; ++i )
			{
				string v = ((CommandEntry)list[i]).Command;

				if ( (sb.Length + 1 + v.Length) >= 256 )
				{
					m.SendAsciiMessage( 0x482, sb.ToString() );
					sb = new StringBuilder();
					sb.Append( v );
				}
				else
				{
					sb.Append( ' ' );
					sb.Append( v );
				}
			}

			if ( sb.Length > 0 )
				m.SendAsciiMessage( 0x482, sb.ToString() );
		}

		[Usage( "SMsg <text>" )]
		[Aliases( "S", "SM" )]
		[Description( "Broadcasts a message to all online staff." )]
		public static void StaffMessage_OnCommand( CommandEventArgs e )
		{
			BroadcastMessage( AccessLevel.Counselor, e.Mobile.SpeechHue, String.Format( "[{0}] {1}", e.Mobile.Name, e.ArgString ) );
		}

		[Usage( "BCast <text>" )]
		[Aliases( "B", "BC" )]
		[Description( "Broadcasts a message to everyone online." )]
		public static void BroadcastMessage_OnCommand( CommandEventArgs e )
		{
			BroadcastMessage( AccessLevel.Player, 0x482, String.Format( "Staff message from {0}:", e.Mobile.Name ) );
			BroadcastMessage( AccessLevel.Player, 0x482, e.ArgString );
		}

		public static void BroadcastMessage ( AccessLevel ac, int hue, string message ) 
		{ 
			foreach ( NetState state in NetState.Instances )
			{
				Mobile m = state.Mobile;

				if ( m != null && m.AccessLevel >= ac )
					m.SendMessage( hue, message );
			}
		}

		private class DeleteTarget : Target
		{
			public DeleteTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					CommandLogging.WriteLine( from, "{0} {1} deleting {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );
					((Item)targeted).Delete();
				}
				else if ( targeted is Mobile && !((Mobile)targeted).Player )
				{
					CommandLogging.WriteLine( from, "{0} {1} deleting {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );
					((Mobile)targeted).Delete();
				}
				else
				{
					from.SendMessage( "That cannot be deleted." );
				}
			}
		}

		[Usage( "Delete" )]
		[Aliases( "Remove" )]
		[Description( "Deletes a targeted item or mobile. Does not delete players." )]
		public static void Delete_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteTarget();
		}

		private class DeleteItemByLayerTarget : Target
		{
			private Layer m_Layer;

			public DeleteItemByLayerTarget( Layer layer ) : base( -1, false, TargetFlags.None )
			{
				m_Layer = layer;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Item item = ((Mobile)targeted).FindItemOnLayer( m_Layer );

					if ( item != null )
					{
						CommandLogging.WriteLine( from, "{0} {1} deleting item on layer {2} of {3}", from.AccessLevel, CommandLogging.Format( from ), m_Layer, CommandLogging.Format( targeted ) );
						item.Delete();
					}
				}
				else
				{
					from.SendMessage( "Target a mobile." );
				}
			}
		}

		[Usage( "ShaveHair" )]
		[Description( "Removes the hair of a targeted mobile." )]
		public static void ShaveHair_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteItemByLayerTarget( Layer.Hair );
		}

		[Usage( "ShaveBeard" )]
		[Description( "Removes the beard of a targeted mobile." )]
		public static void ShaveBeard_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteItemByLayerTarget( Layer.FacialHair );
		}

		private class KillResTarget : Target
		{
			private bool m_Kill;

			public KillResTarget( bool kill ) : base( -1, false, TargetFlags.None )
			{
				m_Kill = kill;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					if ( m_Kill )
					{
						CommandLogging.WriteLine( from, "{0} {1} killing {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );
						m.Kill();
					}
					else
					{
						CommandLogging.WriteLine( from, "{0} {1} resurrecting {2}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ) );

						m.PlaySound( 0x214 );
						m.FixedEffect( 0x376A, 10, 16 );

						m.Resurrect();
					}
				}
				else
				{
					from.SendMessage( "Target a mobile." );
				}
			}
		}

		[Usage( "Kill" )]
		[Description( "Kills a targeted mobile." )]
		public static void Kill_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KillResTarget( true );
		}

		[Usage( "Resurrect" )]
		[Aliases( "Res" )]
		[Description( "Resurrects a targeted ghost." )]
		public static void Resurrect_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KillResTarget( false );
		}

		private class NewXYZTarget : Target
		{
			private int m_X, m_Y, m_Z;
			private bool m_xSet, m_ySet, m_zSet;

			public NewXYZTarget( int x, bool xSet, int y, bool ySet, int z, bool zSet ) : base( -1, false, TargetFlags.None )
			{
				m_X = x; m_xSet = xSet;
				m_Y = y; m_ySet = ySet;
				m_Z = z; m_zSet = zSet;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					Item item = (Item)targeted;

					CommandLogging.WriteLine( from, "{0} {1} setting XYZ of {2} to {3} {4} {5}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ), m_X, m_Y, m_Z );
					item.Location = new Point3D( m_xSet ? m_X : item.X, m_ySet ? m_Y : item.Y, m_zSet ? m_Z : item.Z );
				}
				else if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					CommandLogging.WriteLine( from, "{0} {1} setting XYZ of {2} to {3} {4} {5}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( m ), m_X, m_Y, m_Z );
					m.Location = new Point3D( m_xSet ? m_X : m.X, m_ySet ? m_Y : m.Y, m_zSet ? m_Z : m.Z );
				}
			}
		}

		[Usage( "NewXYZ <x> <y> <z>" )]
		[Description( "Moves a targeted item or mobile to the specified location." )]
		public static void NewXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 3 )
			{
				e.Mobile.Target = new NewXYZTarget( e.GetInt32( 0 ), true, e.GetInt32( 1 ), true, e.GetInt32( 2 ), true );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewXYZ <x> <y> <z>" );
			}
		}

		[Usage( "NewX <x>" )]
		[Description( "Changes the X location of a targeted item or mobile." )]
		public static void NewX_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( e.GetInt32( 0 ), true, 0, false, 0, false );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewX <x>" );
			}
		}

		[Usage( "NewY <y>" )]
		[Description( "Changes the Y location of a targeted item or mobile." )]
		public static void NewY_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( 0, false, e.GetInt32( 0 ), true, 0, false );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewY <y>" );
			}
		}

		[Usage( "NewZ <z>" )]
		[Description( "Changes the Z location of a targeted item or mobile." )]
		public static void NewZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( 0, false, 0, false, e.GetInt32( 0 ), true );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewZ <z>" );
			}
		}

		private class IncXYZTarget : Target
		{
			private int m_xOffset, m_yOffset, m_zOffset;

			public IncXYZTarget( int xOffset, int yOffset, int zOffset ) : base( -1, false, TargetFlags.None )
			{
				m_xOffset = xOffset;
				m_yOffset = yOffset;
				m_zOffset = zOffset;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					Item item = (Item)targeted;

					CommandLogging.WriteLine( from, "{0} {1} offseting XYZ of {2} by {3} {4} {5}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( targeted ), m_xOffset, m_yOffset, m_zOffset );
					item.Location = new Point3D( item.X + m_xOffset, item.Y + m_yOffset, item.Z + m_zOffset );
				}
				else if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					CommandLogging.WriteLine( from, "{0} {1} offseting XYZ of {2} by {3} {4} {5}", from.AccessLevel, CommandLogging.Format( from ), CommandLogging.Format( m ), m_xOffset, m_yOffset, m_zOffset );
					m.Location = new Point3D( m.X + m_xOffset, m.Y + m_yOffset, m.Z + m_zOffset );
				}
			}
		}

		[Usage( "IncXYZ <x> <y> <z>" )]
		[Description( "Offsets the X, Y, and Z location of a targeted item or mobile." )]
		public static void IncXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 3 )
			{
				e.Mobile.Target = new IncXYZTarget( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 2 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncXYZ <x> <y> <z>" );
			}
		}

		[Usage( "IncX <x>" )]
		[Description( "Offsets the X location of a targeted item or mobile." )]
		public static void IncX_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( e.GetInt32( 0 ), 0, 0 );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncX <x>" );
			}
		}

		[Usage( "IncY <y>" )]
		[Description( "Offsets the Y location of a targeted item or mobile." )]
		public static void IncY_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( 0, e.GetInt32( 0 ), 0 );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncY <y>" );
			}
		}

		[Usage( "IncZ <z>" )]
		[Description( "Offsets the Z location of a targeted item or mobile." )]
		public static void IncZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( 0, 0, e.GetInt32( 0 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncZ <z>" );
			}
		}

		[Usage( "AutoPageNotify" )]
		[Aliases( "APN" )]
		[Description( "Toggles your auto-page-notify status." )]
		public static void APN_OnCommand( CommandEventArgs e )
		{
			Mobile m = e.Mobile;

			m.AutoPageNotify = !m.AutoPageNotify;

			m.SendMessage( "Your auto-page-notify has been turned {0}.", m.AutoPageNotify ? "on" : "off" );
		}

		[Usage( "Animate <action> <frameCount> <repeatCount> <forward> <repeat> <delay>" )]
		[Description( "Makes your character do a specified animation." )]
		public static void Animate_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 6 )
			{
				e.Mobile.Animate( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 2 ), e.GetBoolean( 3 ), e.GetBoolean( 4 ), e.GetInt32( 5 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: Animate <action> <frameCount> <repeatCount> <forward> <repeat> <delay>" );
			}
		}

		private class TeleportTarget : Target
		{
			public TeleportTarget() : base( -1, true, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				IPoint3D p = targeted as IPoint3D;

				if ( p != null )
				{
					if ( p is Item )
						p = ((Item)p).GetWorldLocation();

					CommandLogging.WriteLine( from, "{0} {1} teleporting to {2}", from.AccessLevel, CommandLogging.Format( from ), new Point3D( p ) );

					Point3D fromLoc = from.Location;
					Point3D toLoc = new Point3D( p );

					from.Location = toLoc;
					from.ProcessDelta();

					if ( !from.Hidden )
					{
						Effects.SendLocationParticles( EffectItem.Create( fromLoc, from.Map, EffectItem.DefaultDuration ), 0x3728, 10, 10, 2023 );
						Effects.SendLocationParticles( EffectItem.Create(   toLoc, from.Map, EffectItem.DefaultDuration ), 0x3728, 10, 10, 5023 );

						from.PlaySound( 0x1FE );
					}
				}
			}
		}

		[Usage( "Teleport" )]
		[Aliases( "Tele" )]
		[Description( "Teleports your character to a targeted location." )]
		public static void Teleport_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new TeleportTarget();
		}

		private class SetHiddenTarget : Target
		{
			private bool m_Value;

			public SetHiddenTarget( bool value ) : base( 18, false, TargetFlags.None )
			{
				m_Value = value;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					CommandLogging.WriteLine( from, "{0} {1} {2} {3}", from.AccessLevel, CommandLogging.Format( from ), m_Value ? "hiding" : "unhiding", CommandLogging.Format( m ) );

					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z + 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z - 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z + 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z - 4 ), m.Map, 0x3728, 13 );

					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 11 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 7 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 3 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z - 1 ), m.Map, 0x3728, 13 );

					m.PlaySound( 0x228 );
					m.Hidden = m_Value;
				}
			}
		}

		[Usage( "Hide" )]
		[Description( "Makes a targeted mobile disappear in a puff of smoke." )]
		public static void Hide_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetHiddenTarget( true );
		}

		[Usage( "Unhide" )]
		[Description( "Makes a targeted mobile appear in a puff of smoke." )]
		public static void Unhide_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetHiddenTarget( false );
		}

		[Usage( "Cast <name>" )]
		[Description( "Casts a spell by name." )]
		public static void Cast_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				Spell spell = SpellRegistry.NewSpell( e.GetString( 0 ), e.Mobile, null );

				if ( spell != null )
				{
					spell.Cast();
				}
				else
				{
					e.Mobile.SendMessage( "That spell was not found." );
				}
			}
			else
			{
				e.Mobile.SendMessage( "Format: Cast <name>" );
			}
		}

		private class StuckMenuTarget : Target
		{
			public StuckMenuTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					from.SendGump( new StuckMenu( from, (Mobile) targeted, false ) );
				}
			}
		}

		[Usage( "Stuck" )]
		[Description( "Opens a menu of towns, used for teleporting stuck mobiles." )]
		public static void Stuck_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new StuckMenuTarget();
		}

		private class SetBlessedTarget : Target
		{
			private bool m_Value;

			public SetBlessedTarget( bool value ) : base( -1, false, TargetFlags.None )
			{
				m_Value = value;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile targ = (Mobile)targeted;

					if ( from == targ || from.AccessLevel > targ.AccessLevel )
					{
						CommandLogging.LogChangeProperty( from, targ, "Blessed", m_Value.ToString() );
						targ.Blessed = m_Value;
					}
				}
			}
		}

		[Usage( "Immortal" )]
		[Aliases( "Invul" )]
		[Description( "Makes a targeted mobile immortal." )]
		public static void Immortal_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetBlessedTarget( true );
		}

		[Usage( "Mortal" )]
		[Aliases( "NoInvul" )]
		[Description( "Makes a targeted mobile mortal." )]
		public static void Mortal_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetBlessedTarget( false );
		}

		[Usage( "Light <level>")]
		[Description( "Set your local lightlevel." )]
		public static void Light_OnCommand( CommandEventArgs e )
		{
			e.Mobile.LightLevel = e.GetInt32( 0 );
		}

		[Usage( "Stats")]
		[Description( "View some stats about the server." )]
		public static void Stats_OnCommand( CommandEventArgs e )
		{
			e.Mobile.SendMessage( "Open Connections: {0}", Network.NetState.Instances.Count );
			e.Mobile.SendMessage( "Mobiles: {0}", World.Mobiles.Count );
			e.Mobile.SendMessage( "Items: {0}", World.Items.Count );
		}
	}
}