using System;
using System.IO;
using System.Collections;
using Server;

namespace Server.Engines.BulkOrders
{
	public class LargeBulkEntry
	{
		private LargeBOD m_Owner;
		private int m_Amount;
		private SmallBulkEntry m_Details;

		public int Amount{ get{ return m_Amount; } set{ m_Amount = value; m_Owner.InvalidateProperties(); } }
		public SmallBulkEntry Details{ get{ return m_Details; } }

		private static SmallBulkEntry[] m_LargeRing, m_LargePlate, m_LargeChain;

		public static SmallBulkEntry[] LargeRing
		{
			get
			{
				if ( m_LargeRing == null )
					m_LargeRing = LoadEntries( "Blacksmith", "largering" );

				return m_LargeRing;
			}
		}

		public static SmallBulkEntry[] LargePlate
		{
			get
			{
				if ( m_LargePlate == null )
					m_LargePlate = LoadEntries( "Blacksmith", "largeplate" );

				return m_LargePlate;
			}
		}

		public static SmallBulkEntry[] LargeChain
		{
			get
			{
				if ( m_LargeChain == null )
					m_LargeChain = LoadEntries( "Blacksmith", "largechain" );

				return m_LargeChain;
			}
		}

		public static SmallBulkEntry[] LoadEntries( string type, string name )
		{
			return SmallBulkEntry.LoadEntries( type, name );
		}

		public static LargeBulkEntry[] ConvertEntries( LargeBOD owner, SmallBulkEntry[] small )
		{
			LargeBulkEntry[] large = new LargeBulkEntry[small.Length];

			for ( int i = 0; i < small.Length; ++i )
				large[i] = new LargeBulkEntry( owner, small[i] );

			return large;
		}

		public LargeBulkEntry( LargeBOD owner, SmallBulkEntry details )
		{
			m_Owner = owner;
			m_Details = details;
		}

		public LargeBulkEntry( LargeBOD owner, GenericReader reader )
		{
			m_Owner = owner;
			m_Amount = reader.ReadInt();

			Type realType = null;

			string type = reader.ReadString();

			if ( type != null )
			{
				realType = Type.GetType( type, false, true );

				if ( realType == null )
					realType = ScriptCompiler.Assembly.GetType( type, false, true );
			}

			m_Details = new SmallBulkEntry( realType, reader.ReadInt(), reader.ReadInt() );
		}

		public void Serialize( GenericWriter writer )
		{
			writer.Write( m_Amount );
			writer.Write( m_Details.Type == null ? null : m_Details.Type.FullName );
			writer.Write( m_Details.Number );
			writer.Write( m_Details.Graphic );
		}
	}
}