using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public class DefAlchemy : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Alchemy;	}
		}

		public override string GumpTitle
		{
			get { return "ALCHEMY MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 195; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefAlchemy();

				return m_CraftSystem;
			}
		}

		private DefAlchemy() : base( 1, 1, 3.1 )
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else if ( tool.Durability < 1 )
			{
				from.SendMessage( "This tool is broken" );
				return false;
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			from.PlaySound( 0x242 );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}

			if ( failed )
			{
				from.SendLocalizedMessage( 500287 ); // You fail to create a useful potion.

				from.AddToBackpack( new Bottle() );
			}
			else
			{
				from.PlaySound( 0x240 ); // Sound of a filling bottle
				from.SendLocalizedMessage( 500279 ); // You poor the potion into a bottle...
			}
		}

		public override void InitCraftList()
		{
			int index = -1;
			
			// Agility Potion
			index = AddCraft( typeof( AgilityPotion ), "Agility", "Agility", 15.1, 30.0, typeof( Bloodmoss ), "Bloodmoss", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315);
			index = AddCraft( typeof( GreaterAgilityPotion ), "Agility", "Greater Agility", 35.1, 60.0, typeof( Bloodmoss ), "Bloodmoss", 3, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			
			// Cure Potion
			index = AddCraft( typeof( LesserCurePotion ), "Cure", "Lesser Cure", 0.0, 20.0, typeof( Garlic ), "Garlic", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( CurePotion ), "Cure", "Cure", 25.1, 50.0, typeof( Garlic ), "Garlic", 3, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( GreaterCurePotion ), "Cure", "Greater Cure", 65.1, 100.0, typeof( Garlic ), "Garlic", 6, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );

			// Explosion Potion
			index = AddCraft( typeof( LesserExplosionPotion ), "Explosion", "Lesser Explosion", 5.1, 35.0, typeof( SulfurousAsh ), "Sulfurous Ash", 3, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( ExplosionPotion ), "Explosion", "Explosion", 35.1, 65.0, typeof( SulfurousAsh ), "Sulfurous Ash", 5, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( GreaterExplosionPotion ), "Explosion", "Greater Explosion", 65.1, 95.0, typeof( SulfurousAsh ), "Sulfurous Ash", 10, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			
			// Heal Potion
			index = AddCraft( typeof( LesserHealPotion ), "Heal", "Lesser Heal", 0.0, 30.0, typeof( Ginseng ), "Ginseng", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( HealPotion ), "Heal", "Heal", 15.1, 60.0, typeof( Ginseng ), "Ginseng", 3, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( GreaterHealPotion ), "Heal", "Greater Heal", 55.1, 100.0, typeof( Ginseng ), "Ginseng", 6, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			
			// Nightsight Potion
			index = AddCraft( typeof( NightSightPotion ), "Nightsight", "Nightsight", 0.0, 10.0, typeof( SpidersSilk ), "Spiders Silk", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315);

			// Poison Potion
			index = AddCraft( typeof( LesserPoisonPotion ), "Poison", "Lesser Poison", 0.0, 20.0, typeof( Nightshade ), "Nightshade", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( PoisonPotion ), "Poison", "Poison", 15.1, 30.0, typeof( Nightshade ), "Nightshade", 2, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( GreaterPoisonPotion ), "Poison", "Greater Poison", 55.1, 100.0, typeof( Nightshade ), "Nightshade", 4, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( DeadlyPoisonPotion ), "Poison", "Deadly Poison", 90.1, 100.0, typeof( Nightshade ), "Nightshade", 8, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			
			// Refresh Potion
			index = AddCraft( typeof( RefreshPotion ), "Refresh", "Refresh", 15.1, 30.0, typeof( BlackPearl ), "Black Pearl", 1, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( TotalRefreshPotion ), "Refresh", "Total Refresh", 25.1, 60.0, typeof( BlackPearl ), "Black Pearl", 5, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			
			// Strength Potion
			index = AddCraft( typeof( StrengthPotion ), "Strength", "Strength", 25.1, 50.0, typeof( MandrakeRoot ), "Mandrake Root", 2, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
			index = AddCraft( typeof( GreaterStrengthPotion ), "Strength", "Greater Strength", 45.1, 90.0, typeof( MandrakeRoot ), "Mandrake Root", 5, 501904 );
			AddRes( index, typeof ( Bottle ), "Empty Bottle", 1, 500315 );
		}
	}
}