using System;
using Server.Items;
using Server.Mobiles;

namespace Server.Engines.Craft
{
	public class DefInscription : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Inscribe; }
		}

		public override string GumpTitle
		{
			get { return "INSCRIPTION MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 165; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefInscription();

				return m_CraftSystem;
			}
		}

		private DefInscription() : base( 1, 1, 3.0 )
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type typeItem )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else if ( tool.Durability < 1 )
			{
				from.SendMessage( "This tool is broken" );
				return false;
			}
			 
			if(typeItem != null)
			{
				object o = Activator.CreateInstance( typeItem ); 
			 
				if ( o is SpellScroll ) 
				{
					Spellbook spellbook = from.GetSpellbook( typeof( Spellbook ) ) as Spellbook;
					SpellScroll scroll = (SpellScroll)o;

					if ( spellbook != null )
					{
						if ( spellbook.HasSpell( scroll.SpellID ) )
						{
							scroll.Delete();

							return true;
						}
						else
						{
							scroll.Delete();

							from.SendLocalizedMessage( 1042404 ); // You don't have that spell!

							return false;
						}
					}
					else
					{
						scroll.Delete();

						from.SendLocalizedMessage( 501608 ); // You don't have a spellbook.

						return false;
					}
				}
				else
				{
					return true;
				}
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			from.PlaySound( 0x249 );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}
			if ( failed )
			{
				from.SendLocalizedMessage( 501630 ); // You fail to inscribe the scroll, and the scroll is ruined.
			}
			else
			{
				from.SendLocalizedMessage( 501629 ); // You inscribe the spell and put the scroll in your backpack.
			}
		}

		public override void InitCraftList()
		{
			int index = -1;

			//First Circle Spells (4 mana)**************************************************

			//Clumsy - Blood Moss, Nightshade
			index = AddCraft( typeof( ClumsyScroll ), "First Circle", "Clumsy", -25.0, 25.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627  );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Create Food - Garlic, Ginseng, Mandrake Root
			index = AddCraft( typeof( CreateFoodScroll ), "First Circle", "Create Food", -25.0, 25.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Feeblemind - Nightshade, Ginseng
			index = AddCraft( typeof( FeeblemindScroll ), "First Circle", "Feeblemind", -25.0, 25.0, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Heal - Garlic, Ginseng, Spider's Silk
			index = AddCraft( typeof( HealScroll ), "First Circle", "Heal", -25.0, 25.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Magic Arrow - Sulphurous Ash
			index = AddCraft( typeof( MagicArrowScroll ), "First Circle", "Magic Arrow", -25.0, 25.0, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Night Sight - Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( NightSightScroll ), "First Circle", "Night Sight", -25.0, 25.0, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Reactive Armor - Garlic, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( ReactiveArmorScroll ), "First Circle", "Reactive Armor", -25.0, 25.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Weaken - Garlic, Nightshade
			index = AddCraft( typeof( WeakenScroll ), "First Circle", "Weaken", -25.0, 25.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 4 );

			//Second Circle Spells (6 mana)**************************************************

			//Agility - Blood Moss, Mandrake Root
			index = AddCraft( typeof( AgilityScroll ), "Second Circle", "Agility", -15.0, 40.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Cunning - Nightshade, Mandrake Root
			index = AddCraft( typeof( CunningScroll ), "Second Circle", "Cunning", -15.0, 40.0, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Cure - Garlic, Ginseng
			index = AddCraft( typeof( CureScroll ), "Second Circle", "Cure", -15.0, 40.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Harm - Nightshade, Spider's Silk
			index = AddCraft( typeof( HarmScroll ), "Second Circle", "Harm", -15.0, 40.0, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Magic Trap - Garlic, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( MagicTrapScroll ), "Second Circle", "Magic Trap", -15.0, 40.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Magic UnTrap - Blood Moss, Sulphurous Ash
			index = AddCraft( typeof( MagicUnTrapScroll ), "Second Circle", "Magic Untrap", -15.0, 40.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Protection - Garlic, Ginseng, Sulphurous Ash
			index = AddCraft( typeof( ProtectionScroll ), "Second Circle", "Protection", -15.0, 40.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Strength - Mand. Root, Nightshade
			index = AddCraft( typeof( StrengthScroll ), "Second Circle", "Strength", -15.0, 40.0, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 6 );

			//Third Circle Spells (9 mana) *******************************************************

			//Bless - Mandrake Root, Garlic
			index = AddCraft( typeof( BlessScroll ), "Third Circle", "Bless", 00.0, 55.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Fireball - Black Pearl
			index = AddCraft( typeof( FireballScroll ), "Third Circle", "Fireball", 00.0, 55.0, typeof( BlackPearl ), "BlackPearl", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Magic Lock - Bloodmoss, Garlic, Sulphurous Ash
			index = AddCraft( typeof( MagicLockScroll ), "Third Circle", "Magic Lock", 00.0, 55.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Poison - Nightshade
			index = AddCraft( typeof( PoisonScroll ), "Third Circle", "Poison", 00.0, 55.0, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Telekinesis - Blood Moss, Mandrake Root
			index = AddCraft( typeof( TelekinisisScroll ), "Third Circle", "Telekinesis", 00.0, 55.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Teleport - Blood Moss, Mandrake Root
			index = AddCraft( typeof( TeleportScroll ), "Third Circle", "Teleport", 00.0, 55.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Unlock - Blood Moss, Sulphurous Ash
			index = AddCraft( typeof( UnlockScroll ), "Third Circle", "Unlock", 00.0, 55.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Wall of Stone - Blood Moss, Garlic
			index = AddCraft( typeof( WallOfStoneScroll ), "Third Circle", "Wall of Stone", 00.0, 55.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 9 );

			//Fourth Circle Spells (11 mana) *******************************************************

			//Archcure - Garlic, Ginseng, Mandrake Root
			index = AddCraft( typeof( ArchCureScroll ), "Fourth Circle", "Archcure", 20.0, 70.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Archprotection - Garlic, Ginseng, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( ArchProtectionScroll ), "Fourth Circle", "Archprotection", 20.0, 70.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Curse - Garlic, Nightshade, Sulphurous Ash
			index = AddCraft( typeof( CurseScroll ), "Fourth Circle", "Curse", 20.0, 70.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Fire Field - Black Pearl, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( FireFieldScroll ), "Fourth Circle", "Fire Field", 20.0, 70.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Greater Heal - Garlic, Ginseng, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( GreaterHealScroll ), "Fourth Circle", "Greater Heal", 20.0, 70.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Lightning - Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( LightningScroll ), "Fourth Circle", "Lightning", 20.0, 70.0, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Mana Drain - Black Pearl, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( ManaDrainScroll ), "Fourth Circle", "Mana Drain", 20.0, 70.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Recall  - Black Pearl, Blood Moss, Mandrake Root
			index = AddCraft( typeof( RecallScroll ), "Fourth Circle", "Recall", 20.0, 70.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 11 );

			//Fifth Circle Spells (14 mana) *************************************************************

			//Blade Spirits - Black Pearl, Mandrake Root, Nightshade
			index = AddCraft( typeof( BladeSpiritsScroll ), "Fifth Circle", "Blade Spirits", 35.0, 85.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Dispel Field - Black Pearl, Garlic, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( DispelFieldScroll ), "Fifth Circle", "Dispel Field", 35.0, 85.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Incognito - Blood Moss, Garlic, Nightshade
			index = AddCraft( typeof( IncognitoScroll ), "Fifth Circle", "Incognito", 35.0, 85.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Magic Reflection - Garlic, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( MagicReflectScroll ), "Fifth Circle", "Magic Reflection", 35.0, 85.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Mind Blast - Black Pearl, Mandrake Root, Nightshade, Sulphurous Ash
			index = AddCraft( typeof( MindBlastScroll ), "Fifth Circle", "Mind Blast", 35.0, 85.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Paralyze - Garlic, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( ParalyzeScroll ), "Fifth Circle", "Paralyze", 35.0, 85.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Poison Field - Black Pearl, Nightshade, Spider's Silk
			index = AddCraft( typeof( PoisonFieldScroll ), "Fifth Circle", "Poison Field", 35.0, 85.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Summon Creature - Blood Moss, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( SummonCreatureScroll ), "Fifth Circle", "Summon Creature", 35.0, 85.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 14 );

			//Sixth Circle Spells (20 mana)******************************************************************

			//Dispel - Garlic, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( DispelScroll ), "Sixth Circle", "Dispel", 50.0, 100.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Energy Bolt - Black Pearl, Nightshade
			index = AddCraft( typeof( EnergyBoltScroll ), "Sixth Circle", "Energy Bolt", 50.0, 100.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Explosion - Blood Moss, Mandrake Root
			index = AddCraft( typeof( ExplosionScroll ), "Sixth Circle", "Explosion", 50.0, 100.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Invisibility - Blood Moss, Nightshade
			index = AddCraft( typeof( InvisibilityScroll ), "Sixth Circle", "Invisibility", 50.0, 100.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Mark - Black Pearl, Blood Moss, Mandrake Root
			index = AddCraft( typeof( MarkScroll ), "Sixth Circle", "Mark", 50.0, 100.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Mass Curse - Garlic, Mandrake Root, Nightshade, Sulphurous Ash
			index = AddCraft( typeof( MassCurseScroll ), "Sixth Circle", "Mass Curse", 50.0, 100.0, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Paralyze Field - Black Pearl, Ginseng, Spider's Silk
			index = AddCraft( typeof( ParalyzeFieldScroll ), "Sixth Circle", "Paralyze Field", 50.0, 100.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Reveal - Blood Moss, Sulphurous Ash
			index = AddCraft( typeof( RevealScroll ), "Sixth Circle", "Reveal", 50.0, 100.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 20 );

			//Seventh Circle Spells (40 mana) **************************************************
		
			//Chain Lightning - Black Pearl, Blood Moss, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( ChainLightningScroll ), "Seventh Circle", "Chain Lightning", 60.0, 110.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Energy Field - Black Pearl, Mandrake Root, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( EnergyFieldScroll ), "Seventh Circle", "Energy Field", 60.0, 110.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Flamestrike - Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( FlamestrikeScroll ), "Seventh Circle", "Flamestrike", 60.0, 110.0, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Gate Travel - Black Pearl, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( GateTravelScroll ), "Seventh Circle", "Gate Travel", 60.0, 110.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Mana Vampire - Black Pearl, Blood Moss, Mandrake, Spider's Silk
			index = AddCraft( typeof( ManaVampireScroll ), "Seventh Circle", "Mana Vampire", 60.0, 110.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Mass Dispel - Black Pearl, Garlic, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( MassDispelScroll ), "Seventh Circle", "Mass Dispel", 60.0, 110.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Meteor Swarm - Blood Moss, Mandrake Root, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( MeteorSwarmScroll ), "Seventh Circle", "Meteor Swarm", 60.0, 110.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

			//Polymorph - Blood Moss, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( PolymorphScroll ), "Seventh Circle", "Polymorph", 60.0, 110.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 40 );

        
			//Eighth Circle Spells (50 mana) *********************************************************

			//Earthquake - Blood Moss, Ginseng, Mandrake Root, Sulphurous Ash
			index = AddCraft( typeof( EarthquakeScroll ), "Eighth Circle", "Earthquake", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Energy Vortex - Black Pearl, Blood Moss, Mandrake Root, Nightshade
			index = AddCraft( typeof( EnergyVortexScroll ), "Eighth Circle", "Energy Vortex", 75.0, 125.0, typeof( BlackPearl ), "Black Pearl", 1 , 501627 );
			AddRes( index, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( Nightshade ), "Nightshade", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Resurrection - Blood Moss, Garlic, Ginseng
			index = AddCraft( typeof( ResurrectionScroll ), "Eighth Circle", "Resurrection", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( Garlic ), "Garlic", 1 , 501627 );
			AddRes( index, typeof( Ginseng ), "Ginseng", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Summon Air Elem. - Blood Moss, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( SummonAirElementalScroll ), "Eighth Circle", "Summon Air Elemental", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Summon Daemon - Blood Moss, Mandrake Root, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( SummonDaemonScroll ), "Eighth Circle", "Summon Daemon", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Summon Earth Elem. - Blood Moss, Mandrake Root, Spider's Silk
			index = AddCraft( typeof( SummonEarthElementalScroll ), "Eighth Circle", "Summon Earth Elemental", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Summon Fire Elem. - Blood Moss, Mandrake Root, Spider's Silk, Sulphurous Ash
			index = AddCraft( typeof( SummonFireElementalScroll ), "Eighth Circle", "Summon Fire Elemental", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( SulfurousAsh ), "Sulphurous Ash", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			//Summon Water Elem. - Blood Moss, Mandrake Root, Spider's Silk 
			index = AddCraft( typeof( SummonWaterElementalScroll ), "Eighth Circle", "Summon Water Elemental", 75.0, 125.0, typeof( Bloodmoss ), "Blood Moss", 1 , 501627 );
			AddRes( index, typeof( MandrakeRoot ), "Mandrake Root", 1 , 501627 );
			AddRes( index, typeof( SpidersSilk ), "Spider's Silk", 1 , 501627 );
			AddRes( index, typeof( BlankScroll ), "Blank Scroll", 1, 1044378 );
			SetManaReq( index, 50 );

			// Runebook
			index = AddCraft( typeof( Runebook ), "Other", "Runebook", 45.0, 85.0, typeof( BlankScroll ), "Blank Scroll", 8, 1044378 );
			AddRes( index, typeof( RecallScroll ), "Recall Scroll", 1 , "You need a recall scroll to craft this." );
			AddRes( index, typeof( GateTravelScroll ), "Gate Travel Scroll", 1 , "You need a gate travel scroll to craft this." );
		}
	}
}