using System;
using System.IO;
using System.Collections;
using Server;
using Server.Network;
using Server.Gumps;

namespace Server.Engines.Help
{
	public class MessageSentGump : Gump
	{
		private string m_Name, m_Text;
		private Mobile m_Mobile;

		public MessageSentGump( Mobile mobile, string name, string text ) : base( 30, 30 )
		{
			m_Name = name;
			m_Text = text;
			m_Mobile = mobile;

			Closable = false;

			AddPage( 0 );

			AddBackground( 0, 0, 92, 75, 0xA3C );

			AddImageTiled( 5, 7, 82, 61, 0x52 );
			AddAlphaRegion( 5, 7, 82, 61 );

			AddImageTiled( 9, 11, 21, 53, 0xBBC );

			AddButton( 10, 12, 0x7D2, 0x7D2, 0, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 34, 28, 65, 24, 3001002, 0xFFFFFF, false, false  ); // Message
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			m_Mobile.SendMessage( 0x482, "{0} tells you:", m_Name );
			m_Mobile.SendMessage( 0x482, m_Text );
		}
	}

	public class PageQueueGump : Gump
	{
		private PageEntry[] m_List;

		public PageQueueGump() : base( 50, 50 )
		{
			Add( new GumpPage( 0 ) );
			//Add( new GumpBackground( 0, 0, 410, 448, 9200 ) );
			Add( new GumpImageTiled( 0, 0, 410, 448, 0x52 ) );
			Add( new GumpAlphaRegion( 1, 1, 408, 446 ) );

			Add( new GumpLabel( 180, 12, 2100, "Page Queue" ) );

			ArrayList list = PageQueue.List;

			for ( int i = 0; i < list.Count; )
			{
				PageEntry e = (PageEntry)list[i];

				if ( e.Sender.Deleted || e.Sender.NetState == null )
					PageQueue.Remove( e );
				else
					++i;
			}

			m_List = (PageEntry[])list.ToArray( typeof( PageEntry ) );

			if ( m_List.Length > 0 )
			{
				Add( new GumpPage( 1 ) );

				for ( int i = 0; i < m_List.Length; ++i )
				{
					PageEntry e = m_List[i];

					if ( i >= 5 && (i % 5) == 0 )
					{
						Add( new GumpButton( 368, 12, 0xFA5, 0xFA7, 0, GumpButtonType.Page, (i / 5) + 1 ) );
						Add( new GumpLabel( 298, 12, 2100, "Next Page" ) );
						Add( new GumpPage( (i / 5) + 1 ) );
						Add( new GumpButton( 12, 12, 0xFAE, 0xFB0, 0, GumpButtonType.Page, (i / 5) ) );
						Add( new GumpLabel( 48, 12, 2100, "Previous Page" ) );
					}

					string html = String.Format( "[{0}] {1} <basefont color=#{2:X6}>[<u>{3}</u>]</basefont>", e.Type.ToString(), e.Message, e.Handler == null ? 0xFF0000 : 0xFF, e.Handler == null ? "Unhandled" : "Handling" );

					Add( new GumpHtml( 12, 44 + ((i % 5) * 80), 350, 70, html, true, true ) );
					Add( new GumpButton( 370, 44 + ((i % 5) * 80) + 24, 0xFA5, 0xFA7, i + 1, GumpButtonType.Reply, 0 ) );
				}
			}
			else
			{
				Add( new GumpLabel( 12, 44, 2100, "The page queue is empty." ) );
			}
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			if ( info.ButtonID >= 1 && info.ButtonID <= m_List.Length )
			{
				PageEntryGump g = new PageEntryGump( state.Mobile, m_List[info.ButtonID - 1] );

				g.SendTo( state );
			}
		}
	}

	public class PageEntryGump : Gump
	{
		private PageEntry m_Entry;
		private Mobile m_Mobile;

		private static int[] m_AccessLevelHues = new int[]
			{
				2100,
				2122,
				2117,
				2129,
				2415
			};

		public PageEntryGump( Mobile m, PageEntry entry ) : base( 50, 50 )
		{
			m_Mobile = m;
			m_Entry = entry;

			int buttons = 0;

			int bottom = 336;

			AddPage( 0 );

			AddImageTiled( 0, 0, 410, 376, 0x52 );
			AddAlphaRegion( 1, 1, 408, 374 );

			AddPage( 1 );

			AddLabel( 18, 18, 2100, "Sent:" );
			AddLabelCropped( 128, 18, 264, 20, 2100, entry.Sent.ToString() );

			AddLabel( 18, 38, 2100, "Sender:" );
			AddLabelCropped( 128, 38, 264, 20, 2100, String.Format( "{0} {1} [{2}]", entry.Sender.Name, entry.Sender.Location, entry.Sender.Map ) );

			AddButton( 18, bottom - (buttons * 22), 0xFAB, 0xFAD, 8, GumpButtonType.Reply, 0 );
			AddImageTiled( 52, bottom - (buttons * 22) + 1, 340, 20, 0xBBC/*0x2458*/ );
			AddImageTiled( 53, bottom - (buttons * 22) + 2, 338, 18, 0x2426 );
			AddTextEntry( 53, bottom - (buttons++ * 22) + 2, 338, 18, 0x480, 0, "" );

			AddButton( 18, bottom - (buttons * 22), 0xFA5, 0xFA7, 0, GumpButtonType.Page, 2 );
			AddLabel( 52, bottom - (buttons++ * 22), 2100, "Predefined Response" );

			if ( entry.Sender != m )
			{
				AddButton( 18, bottom - (buttons * 22), 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );
				AddLabel( 52, bottom - (buttons++ * 22), 2100, "Go to Sender" );
			}

			AddLabel(  18, 58, 2100, "Handler:" );

			if ( entry.Handler == null )
			{
				AddLabelCropped( 128, 58, 264, 20, 2100, "Unhandled" );

				AddButton( 18, bottom - (buttons * 22), 0xFB1, 0xFB3, 5, GumpButtonType.Reply, 0 );
				AddLabel( 52, bottom - (buttons++ * 22), 2100, "Delete Page" );

				AddButton( 18, bottom - (buttons * 22), 0xFB7, 0xFB9, 4, GumpButtonType.Reply, 0 );
				AddLabel( 52, bottom - (buttons++ * 22), 2100, "Handle Page" );
			}
			else
			{
				AddLabelCropped( 128, 58, 264, 20, m_AccessLevelHues[(int)entry.Handler.AccessLevel], entry.Handler.Name );

				if ( entry.Handler != m )
				{
					AddButton( 18, bottom - (buttons * 22), 0xFA5, 0xFA7, 2, GumpButtonType.Reply, 0 );
					AddLabel( 52, bottom - (buttons++ * 22), 2100, "Go to Handler" );
				}
				else
				{
					AddButton( 18, bottom - (buttons * 22), 0xFA2, 0xFA4, 6, GumpButtonType.Reply, 0 );
					AddLabel( 52, bottom - (buttons++ * 22), 2100, "Abandon Page" );

					AddButton( 18, bottom - (buttons * 22), 0xFB7, 0xFB9, 7, GumpButtonType.Reply, 0 );
					AddLabel( 52, bottom - (buttons++ * 22), 2100, "Page Handled" );
				}
			}

			AddLabel(  18, 78, 2100, "Page Location:" );
			AddLabelCropped( 128, 78, 264, 20, 2100, String.Format( "{0} [{1}]", entry.PageLocation, entry.PageMap ) );

			AddButton( 18, bottom - (buttons * 22), 0xFA5, 0xFA7, 3, GumpButtonType.Reply, 0 );
			AddLabel( 52, bottom - (buttons++ * 22), 2100, "Go to Page Location" );

			AddLabel(  18, 98, 2100, "Page Type:" );
			AddLabelCropped( 128, 98, 264, 20, 2100, entry.Type.ToString() );

			AddLabel(  18, 118, 2100, "Message:" );
			AddHtml(  128, 118, 250, 100, entry.Message, true, true );

			AddPage( 2 );

			string[][] preresp = PreResponses;

			AddButton( 18, 18, 0xFAE, 0xFB0, 0, GumpButtonType.Page, 1 );

			if ( preresp.Length == 0 )
			{
				AddLabel( 52, 18, 2100, "There are no predefined responses." );
			}
			else
			{
				AddLabel( 52, 18, 2100, "Back" );

				for ( int i = 0; i < preresp.Length; ++i )
				{
					AddButton( 18, 40 + (i * 22), 0xFA5, 0xFA7, 100 + i, GumpButtonType.Reply, 0 );
					AddLabel( 52, 40 + (i * 22), 2100, preresp[i][0] );
				}
			}
		}

		private static string[][] m_PreResponses;

		public static string[][] PreResponses
		{
			get
			{
				if ( m_PreResponses == null )
					m_PreResponses = LoadPreResponses();

				return m_PreResponses;
			}
		}

		public static string[][] LoadPreResponses()
		{
			string path = Path.Combine( Core.BaseDirectory, "Data/pageresponse.cfg" );
			ArrayList list = new ArrayList();

			if ( File.Exists( path ) )
			{
				using ( StreamReader ip = new StreamReader( path ) )
				{
					string line;

					while ( (line = ip.ReadLine()) != null )
					{
						try
						{
							line = line.Trim();

							if ( line.Length == 0 || line.StartsWith( "#" ) )
								continue;

							string[] split = line.Split( '\t' );

							if ( split.Length == 2 )
								list.Add( split );
						}
						catch
						{
						}
					}
				}
			}

			return (string[][])list.ToArray( typeof( string[] ) );
		}

		public void Resend( NetState state )
		{
			PageEntryGump g = new PageEntryGump( m_Mobile, m_Entry );

			g.SendTo( state );
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			switch ( info.ButtonID )
			{
				case 0: // close
				{
					if ( m_Entry.Handler != state.Mobile )
					{
						PageQueueGump g = new PageQueueGump();

						g.SendTo( state );
					}

					break;
				}
				case 1: // go to sender
				{
					Mobile m = state.Mobile;

					if ( m_Entry.Sender.Deleted )
					{
						m.SendMessage( "That character no longer exists." );
					}
					else if ( m_Entry.Sender.Map == null || m_Entry.Sender.Map == Map.Internal )
					{
						m.SendMessage( "That character is not in the world." );
					}
					else
					{
						m.Map = m_Entry.Sender.Map;
						m.Location = m_Entry.Sender.Location;

						m.SendMessage( "You have been teleported to that pages sender." );

						Resend( state );
					}

					break;
				}
				case 2: // go to handler
				{
					Mobile m = state.Mobile;
					Mobile h = m_Entry.Handler;

					if ( h != null )
					{
						if ( h.Deleted )
						{
							m.SendMessage( "That character no longer exists." );
						}
						else if ( h.Map == null || h.Map == Map.Internal )
						{
							m.SendMessage( "That character is not in the world." );
						}
						else
						{
							m.Map = h.Map;
							m.Location = h.Location;

							m.SendMessage( "You have been teleported to that pages handler." );
							Resend( state );
						}
					}
					else
					{
						m.SendMessage( "Nobody is handling that page." );
						Resend( state );
					}

					break;
				}
				case 3: // go to page location
				{
					Mobile m = state.Mobile;

					if ( m_Entry.PageMap == null || m_Entry.PageMap == Map.Internal )
					{
						m.SendMessage( "That location is not in the world." );
					}
					else
					{
						m.Map = m_Entry.PageMap;
						m.Location = m_Entry.PageLocation;

						state.Mobile.SendMessage( "You have been teleported to the original page location." );

						Resend( state );
					}

					break;
				}
				case 4: // handle page
				{
					if ( m_Entry.Handler == null )
					{
						m_Entry.Handler = state.Mobile;

						state.Mobile.SendMessage( "You are now handling the page." );
					}
					else
					{
						state.Mobile.SendMessage( "Someone is already handling that page." );
					}

					Resend( state );

					break;
				}
				case 5: // delete page
				{
					if ( m_Entry.Handler == null )
					{
						PageQueue.Remove( m_Entry );

						state.Mobile.SendMessage( "You delete the page." );

						PageQueueGump g = new PageQueueGump();

						g.SendTo( state );
					}
					else
					{
						state.Mobile.SendMessage( "Someone is handling that page, it can not be deleted." );

						Resend( state );
					}

					break;
				}
				case 6: // abandon page
				{
					if ( m_Entry.Handler == state.Mobile )
					{
						state.Mobile.SendMessage( "You abandon the page." );

						m_Entry.Handler = null;
					}
					else
					{
						state.Mobile.SendMessage( "You are not handling that page." );
					}

					Resend( state );

					break;
				}
				case 7: // page handled
				{
					if ( m_Entry.Handler == state.Mobile )
					{
						PageQueue.Remove( m_Entry );

						m_Entry.Handler = null;

						state.Mobile.SendMessage( "You mark the page as handled, and remove it from the queue." );

						PageQueueGump g = new PageQueueGump();

						g.SendTo( state );
					}
					else
					{
						state.Mobile.SendMessage( "You are not handling that page." );

						Resend( state );
					}

					break;
				}
				case 8: // Send message
				{
					TextRelay text = info.GetTextEntry( 0 );

					if ( text != null )
					{
						m_Entry.Sender.SendGump( new MessageSentGump( m_Entry.Sender, state.Mobile.Name, text.Text ) );
						//m_Entry.Sender.SendMessage( 0x482, "{0} tells you:", state.Mobile.Name );
						//m_Entry.Sender.SendMessage( 0x482, text.Text );
					}

					Resend( state );

					break;
				}
				default:
				{
					int index = info.ButtonID - 100;
					string[][] preresp = PreResponses;

					if ( index >= 0 && index < preresp.Length )
						m_Entry.Sender.SendGump( new MessageSentGump( m_Entry.Sender, state.Mobile.Name, preresp[index][1] ) );

					Resend( state );

					break;
				}
			}
		}
	}
}