using System;
using System.Collections;
using Server.Network;

namespace Server.Items
{
	public abstract class BaseArmor : Item, IArmor
	{
		private int m_ArBaseValue;
		private int m_MaxHitPoints;
		private int m_HitPoints;
		private Mobile m_Crafter;
		private ArmorQuality m_Quality;
		private ArmorDurabilityLevel m_Durability;
		private ArmorProtectionLevel m_Protection;
		private ArmorMaterialType m_Material;
		private ArmorBodyType m_BodyPos;
		private CraftResource m_Resource;
		private int m_StrBonus;
		private int m_DexBonus;
		private int m_IntBonus;
		private int m_StrReq;
		private int m_DexReq;
		private int m_IntReq;
		private ArmorMeditationAllowance m_Meditate;
		private bool m_Identified;

		private static double[] m_ArmorScalars = { 0.07, 0.07, 0.14, 0.15, 0.22, 0.35 };

		public static double[] ArmorScalars
		{
			get
			{
				return m_ArmorScalars;
			}
			set
			{
				m_ArmorScalars = value;
			}
		}

		public static void ValidateMobile( Mobile m )
		{
			for ( int i = m.Items.Count - 1; i >= 0; --i )
			{
				if ( i >= m.Items.Count )
					continue;

				Item item = (Item)m.Items[i];

				if ( item is BaseArmor )
				{
					BaseArmor armor = (BaseArmor)item;

					if ( !armor.AllowMaleWearer && m.Body.IsMale && m.AccessLevel < AccessLevel.GameMaster )
					{
						if ( armor.AllowFemaleWearer )
							m.SendLocalizedMessage( 1010388 ); // Only females can wear this.
						else
							m.SendMessage( "You may not wear this." );

						m.AddToBackpack( armor );
					}
					else if ( !armor.AllowFemaleWearer && m.Body.IsFemale && m.AccessLevel < AccessLevel.GameMaster )
					{
						if ( armor.AllowMaleWearer )
							m.SendMessage( "Only males can wear this." );
						else
							m.SendMessage( "You may not wear this." );

						m.AddToBackpack( armor );
					}
				}
			}
		}

		public virtual bool AllowMaleWearer{ get{ return true; } }
		public virtual bool AllowFemaleWearer{ get{ return true; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorBodyType BodyPosition
		{
			get
			{
				return m_BodyPos;
			} 
			set
			{
				if ( m_BodyPos != value )
				{
					m_BodyPos = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorMaterialType MaterialType
		{
			get
			{
				return m_Material;
			}
			set
			{
				if ( m_Material != value )
				{
					m_Material = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Identified
		{
			get
			{
				return m_Identified;
			}
			set
			{
				m_Identified = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public CraftResource Resource
		{
			get
			{
				return m_Resource;
			}
			set
			{
				if ( m_Resource != value )
				{
					m_Resource = value;
					Hue = CraftResources.GetHue( m_Resource );

					Invalidate();
					InvalidateProperties();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrBonus
		{
			get
			{
				return m_StrBonus;
			}
			set
			{
				m_StrBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int DexBonus
		{
			get
			{
				return m_DexBonus;
			}
			set
			{
				m_DexBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int IntBonus
		{
			get
			{
				return m_IntBonus;
			}
			set
			{
				m_IntBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrRequirement
		{
			get
			{
				return m_StrReq;
			}
			set
			{
				if ( m_StrReq != value )
				{
					m_StrReq = value;
					InvalidateProperties();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int DexRequirement
		{
			get
			{
				return m_DexReq;
			}
			set
			{
				m_DexReq = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int IntRequirement
		{
			get
			{
				return m_IntReq;
			}
			set
			{
				m_IntReq = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseArmorRating
		{
			get
			{
				return m_ArBaseValue;
			}
			set
			{
				if ( m_ArBaseValue != value )
				{
					m_ArBaseValue = value;

					Invalidate();
				}
			}
		}

		public virtual double ArmorScalar
		{
			get
			{
				int pos = (int)m_BodyPos;

				if ( pos >= 0 && pos < m_ArmorScalars.Length )
					return m_ArmorScalars[pos];

				return 1.0;
			}
		}

		public override void OnAdded( object parent )
		{
			if ( parent is Mobile )
				((Mobile)parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
		}

		public virtual double ArmorRating
		{
			get
			{
				int ar = m_ArBaseValue;

				if ( m_Protection != ArmorProtectionLevel.Regular )
					ar += 10 + (5 * (int)m_Protection);

				switch ( m_Resource )
				{
					case CraftResource.DullCopper:		ar += 2; break;
					case CraftResource.ShadowIron:		ar += 4; break;
					case CraftResource.Copper:			ar += 6; break;
					case CraftResource.Bronze:			ar += 8; break;
					case CraftResource.Gold:			ar += 10; break;
					case CraftResource.Agapite:			ar += 12; break;
					case CraftResource.Verite:			ar += 14; break;
					case CraftResource.Valorite:		ar += 16; break;
					case CraftResource.SpinedLeather:	ar += 10; break;
					case CraftResource.HornedLeather:	ar += 13; break;
					case CraftResource.BarbedLeather:	ar += 16; break;
				}

				ar += -8 + (8 * (int)m_Quality);

				return ar * ArmorScalar;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int MaxHitPoints
		{
			get
			{
				return m_MaxHitPoints;
			}
			set 
			{
				m_MaxHitPoints = value;
				InvalidateProperties();
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public int HitPoints
		{
			get 
			{
				return m_HitPoints;
			}
			set 
			{
				if ( value != m_HitPoints && MaxHitPoints != 0 )
				{
					m_HitPoints = value;

					if ( m_HitPoints < 0 )
						Delete();
					else if ( m_HitPoints > MaxHitPoints )
						m_HitPoints = MaxHitPoints;

					InvalidateProperties();
				}
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Crafter
		{
			get
			{
				return m_Crafter;
			}
			set
			{
				if ( m_Crafter != value )
				{
					m_Crafter = value;
					InvalidateProperties();
				}
			}
		}

		
		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorQuality Quality
		{
			get
			{
				return m_Quality;
			}
			set
			{
				if ( m_Quality != value )
				{
					m_Quality = value;

					Invalidate();
					InvalidateProperties();
				}
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorDurabilityLevel Durability
		{
			get
			{
				return m_Durability;
			}
			set
			{
				m_Durability = value;
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorProtectionLevel ProtectionLevel
		{
			get
			{
				return m_Protection;
			}
			set
			{
				if ( m_Protection != value )
				{
					m_Protection = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorMeditationAllowance MeditationAllowance
		{
			get
			{
				return m_Meditate;
			}
			set
			{
				m_Meditate = value;
			}
		}

		protected void Invalidate()
		{
			if ( Parent is Mobile )
				((Mobile)Parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
		}

		public BaseArmor( Serial serial ) :  base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 2 );//version

			writer.Write( (bool) m_Identified );

			writer.Write( (int)m_ArBaseValue );
			writer.Write( (int)m_MaxHitPoints );
			writer.Write( (int)m_HitPoints );
			writer.Write( m_Crafter );
			writer.Write( (int)m_Quality );
			writer.Write( (int)m_Durability );
			writer.Write( (int)m_Protection );
			writer.Write( (int)m_Material );
			writer.Write( (int)m_BodyPos );
			writer.Write( (int)m_Resource );
			//writer.Write( (int)m_Ore.Level );
			writer.Write( (int)m_StrBonus );
			writer.Write( (int)m_DexBonus );
			writer.Write( (int)m_IntBonus );
			writer.Write( (int)m_StrReq );
			writer.Write( (int)m_DexReq );
			writer.Write( (int)m_IntReq );
			writer.Write( (int)m_Meditate );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 2:
				case 1:
					m_Identified = reader.ReadBool();
					goto case 0;
				case 0:
					m_ArBaseValue = reader.ReadInt();
					m_MaxHitPoints = reader.ReadInt();
					m_HitPoints = reader.ReadInt();
					m_Crafter = reader.ReadMobile();
					m_Quality = (ArmorQuality)reader.ReadInt();
					m_Durability = (ArmorDurabilityLevel)reader.ReadInt();
					m_Protection = (ArmorProtectionLevel)reader.ReadInt();
					m_Material = (ArmorMaterialType)reader.ReadInt();
					m_BodyPos = (ArmorBodyType)reader.ReadInt();

					if ( version >= 2 )
					{
						m_Resource = (CraftResource)reader.ReadInt();
					}
					else
					{
						OreInfo info;

						switch ( reader.ReadInt() )
						{
							default:
							case 0: info = OreInfo.Iron; break;
							case 1: info = OreInfo.DullCopper; break;
							case 2: info = OreInfo.ShadowIron; break;
							case 3: info = OreInfo.Copper; break;
							case 4: info = OreInfo.Bronze; break;
							case 5: info = OreInfo.Gold; break;
							case 6: info = OreInfo.Agapite; break;
							case 7: info = OreInfo.Verite; break;
							case 8: info = OreInfo.Valorite; break;
						}

						m_Resource = CraftResources.GetFromOreInfo( info, m_Material );
					}

					m_StrBonus = reader.ReadInt();
					m_DexBonus = reader.ReadInt();
					m_IntBonus = reader.ReadInt();
					m_StrReq = reader.ReadInt();
					m_DexReq = reader.ReadInt();
					m_IntReq = reader.ReadInt();
					m_Meditate = (ArmorMeditationAllowance)reader.ReadInt();

					if ( this.Parent is Mobile && UseStatMods )
					{
						Mobile m = (Mobile)this.Parent;

						string modName = this.Serial.ToString();

						if ( m_StrBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Str, modName + "Str", m_StrBonus, TimeSpan.Zero ) );

						if ( m_DexBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Dex, modName + "Dex", m_DexBonus, TimeSpan.Zero ) );

						if ( m_IntBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Int, modName + "Int", m_IntBonus, TimeSpan.Zero ) );
					}

					break;
			}
		}

		public BaseArmor( int itemID, int BaseRating, ArmorMaterialType Material, ArmorBodyType position ) :  base( itemID )
		{
			m_ArBaseValue = BaseRating;
			m_Material = Material;

			if ( m_Material > ArmorMaterialType.Leather )
				m_Meditate = ArmorMeditationAllowance.None;

			m_BodyPos = position;

			m_Quality = ArmorQuality.Regular;
			m_Durability = ArmorDurabilityLevel.Regular;
			m_Crafter = null;

			this.Layer = (Layer)ItemData.Quality;
		}

		public override bool CanEquip( Mobile from )
		{
			if ( !AllowMaleWearer && from.Body.IsMale && from.AccessLevel < AccessLevel.GameMaster )
			{
				if ( AllowFemaleWearer )
					from.SendLocalizedMessage( 1010388 ); // Only females can wear this.
				else
					from.SendMessage( "You may not wear this." );

				return false;
			}
			else if ( !AllowFemaleWearer && from.Body.IsFemale && from.AccessLevel < AccessLevel.GameMaster )
			{
				if ( AllowMaleWearer )
					from.SendMessage( "Only males can wear this." );
				else
					from.SendMessage( "You may not wear this." );

				return false;
			}
			else if ( UseStatMods )
			{
				if ( from.Dex < m_DexReq || from.Dex+m_DexBonus < 10 )
				{
					from.SendLocalizedMessage( 502077 ); // You do not have enough dexterity to equip this item.
					return false;
				} 
				else if ( from.Str < m_StrReq || from.Str+m_StrBonus < 10 )
				{
					from.SendLocalizedMessage( 500213 ); // You are not strong enough to equip that.
					return false;
				}
				else if ( from.Int < m_IntReq || from.Int+m_IntBonus < 10 )
				{
					from.SendMessage( "You are not smart enough to equip that." );
					return false;
				}
			}

			return base.CanEquip( from );
		}

		public static bool UseStatMods{ get{ return !Core.AOS; } }

		public override bool OnEquip( Mobile from )
		{
			if ( UseStatMods )
			{
				string modName = this.Serial.ToString();

				if ( m_StrBonus != 0 )
					from.AddStatMod( new StatMod( StatType.Str, modName + "Str", m_StrBonus, TimeSpan.Zero ) );

				if ( m_DexBonus != 0 )
					from.AddStatMod( new StatMod( StatType.Dex, modName + "Dex", m_DexBonus, TimeSpan.Zero ) );

				if ( m_IntBonus != 0 )
					from.AddStatMod( new StatMod( StatType.Int, modName + "Int", m_IntBonus, TimeSpan.Zero ) );
			}

			return base.OnEquip( from );
		}

		public override void OnRemoved( object parent )
		{
			if ( parent is Mobile )
			{
				Mobile m = (Mobile)parent;
				string modName = this.Serial.ToString();

				m.RemoveStatMod( modName + "Str" );
				m.RemoveStatMod( modName + "Dex" );
				m.RemoveStatMod( modName + "Int" );

				((Mobile)parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
			}

			base.OnRemoved( parent );
		}

		public virtual int OnHit( BaseWeapon weapon, int damageTaken )
		{
			int ArDmg = 0;
			double HalfAr = ArmorRating / 2.0;
			int Absorbed = (int)(HalfAr + HalfAr*Utility.RandomDouble());

			if ( Absorbed > 1 )
			{
				if ( ( m_Material == ArmorMaterialType.Plate && weapon.Type == WeaponType.Bashing ) ||
					( ( m_Material == ArmorMaterialType.Cloth || m_Material == ArmorMaterialType.Leather ) && weapon.Type == WeaponType.Slashing ) )
				{
					ArDmg = Absorbed / 10;
				} 
				else
				{
					ArDmg = Utility.Random( 2 );
					if ( Absorbed > BaseArmorRating / 2 )
						ArDmg++;//extra point for a lot of damage
				}

				HitPoints -= ArDmg;
			}

			damageTaken -= Absorbed;
			if ( damageTaken < 0 ) 
				damageTaken = 0;

			return damageTaken;
		}

		public override void AddNameProperty( ObjectPropertyList list )
		{
			int number;

			switch ( m_Resource )
			{
				case CraftResource.DullCopper:		number = 1062031; break; // dull copper ~1_val~
				case CraftResource.ShadowIron:		number = 1062032; break; // shadow iron ~1_val~
				case CraftResource.Copper:			number = 1062033; break; // copper ~1_val~
				case CraftResource.Bronze:			number = 1062034; break; // bronze ~1_val~
				case CraftResource.Gold:			number = 1062035; break; // golden ~1_val~
				case CraftResource.Agapite:			number = 1062036; break; // agapite ~1_val~
				case CraftResource.Verite:			number = 1062037; break; // verite ~1_val~
				case CraftResource.Valorite:		number = 1062038; break; // valorite ~1_val~
				case CraftResource.SpinedLeather:	number = 1062082; break; // blue ~1_val~
				case CraftResource.HornedLeather:	number = 1062077; break; // red ~1_val~
				case CraftResource.BarbedLeather:	number = 1062080; break; // green ~1_val~
				default: number = 0; break;
			}

			if ( number > 0 )
				list.Add( number, Name == null ? "#" + LabelNumber.ToString() : Name );
			else if ( Name != null )
				list.Add( Name );
			else
				list.Add( LabelNumber );
		}

		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );

			if ( m_Quality == ArmorQuality.Exceptional )
				list.Add( 1060636 ); // exceptional

			if ( m_Crafter != null )
				list.Add( 1050043, m_Crafter.Name ); // crafted by ~1_NAME~

			if ( m_StrReq > 0 )
				list.Add( 1061170, m_StrReq.ToString() ); // strength requirement ~1_val~

			if ( m_HitPoints > 0 && m_MaxHitPoints > 0 )
				list.Add( 1060639, "{0}\t{1}", m_HitPoints, m_MaxHitPoints ); // durability ~1_val~ / ~2_val~
		}

		public override void OnSingleClick( Mobile from )
		{
			ArrayList attrs = new ArrayList();

			if ( m_Quality == ArmorQuality.Exceptional )
				attrs.Add( new EquipInfoAttribute( 1018305 - (int)m_Quality ) );

			if ( m_Identified )
			{
				if ( m_Durability != ArmorDurabilityLevel.Regular )
					attrs.Add( new EquipInfoAttribute( 1038000 + (int)m_Durability ) );

				if ( m_Protection > ArmorProtectionLevel.Regular && m_Protection <= ArmorProtectionLevel.Invulnerability )
					attrs.Add( new EquipInfoAttribute( 1038005 + (int)m_Protection ) );
			}
			else if ( m_Durability != ArmorDurabilityLevel.Regular || (m_Protection > ArmorProtectionLevel.Regular && m_Protection <= ArmorProtectionLevel.Invulnerability) )
			{
				attrs.Add( new EquipInfoAttribute( 1038000 ) ); // Unidentified
			}

			EquipmentInfo eqInfo = new EquipmentInfo( LabelNumber, m_Crafter, false, (EquipInfoAttribute[])attrs.ToArray( typeof( EquipInfoAttribute ) ) );

			from.Send( new DisplayEquipmentInfo( this, eqInfo ) );
		}
	}
}
