using System;

namespace Server.Items
{
	public abstract class Beverage : Item
	{
		private Mobile m_Poisoner;
		private Poison m_Poison;
		private int m_AlcoholFactor;
		private int m_FillFactor;

		public virtual Item EmptyItem{ get { return null; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Poisoner
		{
			get { return m_Poisoner; }
			set { m_Poisoner = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Poison Poison
		{
			get { return m_Poison; }
			set { m_Poison = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int FillFactor
		{
			get { return m_FillFactor; }
			set { m_FillFactor = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int AlcoholFactor
		{
			get { return m_AlcoholFactor; }
			set { m_AlcoholFactor = value; }
		}

		public Beverage( int itemID ) : base( itemID )
		{
			this.FillFactor = 1;
		}

		public Beverage( Serial serial ) : base( serial )
		{
		}

		public void Drink( Mobile from )
		{
			if ( FillThirst( from, m_FillFactor ) && ( FillBAC( from, m_AlcoholFactor ) ) )
			{
				// Play a random "drink" sound
				from.PlaySound( Utility.Random( 0x30, 2 ) );

				if ( from.Body.IsHuman && !from.Mounted )
					from.Animate( 34, 5, 1, true, false, 0 );

				if ( m_Poison != null )
					from.ApplyPoison( m_Poisoner, m_Poison );

				this.Consume();

				Item item = EmptyItem;

				if ( item != null )
					from.AddToBackpack( item );
			}
		}

		static public bool FillThirst( Mobile from, int fillFactor )
		{
			if ( from.Thirst >= 20 )
			{
				from.SendMessage( "You are simply too full to drink any more!" );
				return false;
			}
			
			int iThirst = from.Thirst + fillFactor;
			if ( iThirst >= 20 )
			{
				from.Thirst = 20;
				from.SendMessage( "You manage to drink the beverage, but you are stuffed!" );
			}
			else
			{
				from.Thirst = iThirst;

				if ( iThirst < 5 )
					from.SendMessage( "You drink the beverage, but are still extremely thirsty." );
				else if ( iThirst < 10 )
					from.SendMessage( "You drink the beverage, and begin to feel more satiated." );
				else if ( iThirst < 15 )
					from.SendMessage( "After drinking the beverage, you feel much less thirsty." );
				else
					from.SendMessage( "You feel quite full after consuming the beverage." );
			}
			return true;
		}

		static public bool FillBAC( Mobile from, int alcoholFactor )
		{
			from.BAC += alcoholFactor;
			
			return true;
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( !Movable )
				return;

			if ( from.InRange( this.GetWorldLocation(), 1 ) )
				Drink( from );
		}

        public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( m_Poisoner );

			Poison.Serialize( m_Poison, writer );
			writer.Write( m_FillFactor );
			writer.Write( m_AlcoholFactor );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_Poisoner = reader.ReadMobile();

					goto case 0;
				}
				case 0:
				{
					m_Poison = Poison.Deserialize( reader );
					m_FillFactor = reader.ReadInt();
					m_AlcoholFactor = reader.ReadInt();
					break;
				}
			}
		}
	}

	[FlipableAttribute( 0x1f7d, 0x1f7e, 0x1f7f, 0x1f80 )]
	public class GlassCider : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public GlassCider() : base( 0x1f7d )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassCider( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f85, 0x1f86, 0x1f87, 0x1f88 )]
	public class GlassLiquor : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public GlassLiquor() : base( 0x1f85 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f89, 0x1f8a, 0x1f8b, 0x1f8c )]
	public class GlassMilk : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public GlassMilk() : base( 0x1f89 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
		}

		public GlassMilk( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f8d, 0x1f8e, 0x1f8f, 0x1f90 )]
	public class GlassWine : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public GlassWine() : base( 0x1f8d )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f91, 0x1f92, 0x1f93, 0x1f94 )]
	public class GlassWater : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public GlassWater() : base( 0x1f91 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
		}

		public GlassWater( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f95, 0x1f96 )]
	public class PitcherAle : Beverage
	{
		public override Item EmptyItem{ get { return new GlassPitcher(); } }

		[Constructable]
		public PitcherAle() : base( 0x1f95 )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 3;
		}

		public PitcherAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class PitcherMilk : Beverage
	{
		public override Item EmptyItem{ get { return new GlassPitcher(); } }

		[Constructable]
		public PitcherMilk() : base( 0x9ad )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
		}

		public PitcherMilk( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f97, 0x1f98 )]
	public class PitcherCider : Beverage
	{
		public override Item EmptyItem{ get { return new GlassPitcher(); } }

		[Constructable]
		public PitcherCider() : base( 0x1f97 )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 5;
		}

		public PitcherCider( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f99, 0x1f9a )]
	public class PitcherLiquor : Beverage
	{
		public override Item EmptyItem{ get { return new GlassPitcher(); } }

		[Constructable]
		public PitcherLiquor() : base( 0x1f99 )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 5;
		}

		public PitcherLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f9b, 0x1f9c )]
	public class PitcherWine : Beverage
	{
		public override Item EmptyItem{ get { return new GlassPitcher(); } }

		[Constructable]
		public PitcherWine() : base( 0x1f9b )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 5;
		}

		public PitcherWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x9ee, 0x9ef )]
	public class MugAle : Beverage
	{
		public override Item EmptyItem{ get { return new Glass(); } }

		[Constructable]
		public MugAle() : base( 0x9ee )
		{
			this.Weight = 1.0;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public MugAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleWine : Beverage
	{
		[Constructable]
		public BottleWine() : base( 0x9c7 )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 5;
		}

		public BottleWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleAle : Beverage
	{
		[Constructable]
		public BottleAle() : base( 0x99f )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 3;
		}

		public BottleAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleLiquor : Beverage
	{
		[Constructable]
		public BottleLiquor() : base( 0x99b )
		{
			this.Weight = 1.0;
			this.FillFactor = 3;
			this.AlcoholFactor = 5;
		}

		public BottleLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}
