using System;
using Server;

namespace Server.Misc
{
	public class NameVerification
	{
		public static readonly char[] SpaceDashPeriodQuote = new char[]
			{
				' ', '-', '.', '\''
			};

		public static readonly char[] Empty = new char[0];

		public static bool Validate( string name, int minLength, int maxLength, bool allowLetters, bool allowDigits, bool noExceptionsAtStart, int maxExceptions, char[] exceptions )
		{
			if ( name.Length < minLength || name.Length > maxLength )
				return false;

			int exceptCount = 0;

			name = name.ToLower();

			for ( int i = 0; i < name.Length; ++i )
			{
				char c = name[i];

				if ( c >= 'a' && c <= 'z' )
				{
					if ( !allowLetters )
						return false;

					exceptCount = 0;
				}
				else if ( c >= '0' && c <= '9' )
				{
					if ( !allowDigits )
						return false;

					exceptCount = 0;
				}
				else
				{
					bool except = false;

					for ( int j = 0; !except && j < exceptions.Length; ++j )
						if ( c == exceptions[j] )
							except = true;

					if ( !except || (i == 0 && noExceptionsAtStart) )
						return false;

					if ( exceptCount++ == maxExceptions )
						return false;
				}
			}

			for ( int i = 0; i < m_Disallowed.Length; ++i )
				if ( name.IndexOf( m_Disallowed[i] ) >= 0 )
					return false;

			for ( int i = 0; i < m_StartDisallowed.Length; ++i )
				if ( name.StartsWith( m_StartDisallowed[i] ) )
					return false;

			return true;
		}

		private static string[] m_StartDisallowed = new string[]
			{
				"seer",
				"counselor",
				"gm",
				"lady",
				"lord"
			};

		private static string[] m_Disallowed = new string[]
			{
				"jigaboo",
				"chigaboo",
				"wop",
				"kyke",
				"kike",
				"tit",
				"spic",
				"prick",
				"piss",
				"lezbo",
				"lesbo",
				"felatio",
				"dyke",
				"dildo",
				"chinc",
				"chink",
				"cunnilingus",
				"cum",
				"cocksucker",
				"cock",
				"clitoris",
				"clit",
				"ass",
				"hitler",
				"penis",
				"nigga",
				"nigger",
				"klit",
				"kunt",
				"jiz",
				"jism",
				"jerkoff",
				"jackoff",
				"goddamn",
				"fag",
				"blowjob",
				"bitch",
				"asshole",
				"dick",
				"pussy",
				"snatch",
				"cunt",
				"twat",
				"shit",
				"fuck",
				"tailor",
				"smith",
				"scholar",
				"rogue",
				"novice",
				"neophyte",
				"merchant",
				"medium",
				"master",
				"mage",
				"lb",
				"journeyman",
				"grandmaster",
				"fisherman",
				"expert",
				"chef",
				"carpenter",
				"british",
				"blackthorne",
				"blackthorn",
				"beggar",
				"archer",
				"apprentice",
				"adept",
				"gamemaster",
				"frozen",
				"squelched",
				"invulnerable",
				"osi",
				"origin"
			};
	}
}