using System;
using System.Collections;
using Server.Items;
using Server.Targeting;
using Server.GenerateRandomMagicItems;

namespace Server.Mobiles
{
	[CorpseName( "a cyclops corpse" )]
	public class Cyclops : BaseCreature
	{
		[Constructable]
		public Cyclops() : base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.2, 0.4 )
		{
			Name = "a cyclops";
			Body = 75;
			BaseSoundID = 604;

			SetStr( 336, 385 );
			SetDex( 96, 115 );
			SetInt( 31, 55 );

			SetHits( 202, 231 );
			SetMana( 0 );

			SetDamage( 7, 23 );

			SetSkill( SkillName.MagicResist, 60.3, 105.0 );
			SetSkill( SkillName.Tactics, 80.1, 100.0 );
			SetSkill( SkillName.Wrestling, 80.1, 90.0 );

			Fame = 4500;
			Karma = -4500;

			VirtualArmor = 48;

			PackGem();

			switch ( Utility.Random( 6 ))
			{
				case 0: PackItem( new AgilityPotion() ); break;
				case 1: PackItem( new LesserCurePotion() ); break;
				case 2: PackItem( new LesserHealPotion() ); break;
				case 3: PackItem( new LesserPoisonPotion() ); break;
				case 4: PackItem( new RefreshPotion() ); break;
				case 5: PackItem( new StrengthPotion() ); break;
			}

			PackItem( new Arrow( 10 ) );
			PackGold( 450, 650 );

			// 30% chance to get magic armor, then about 15% chance to get magic weapon
			if ( 0.3 >= Utility.RandomDouble() ) 
			{
				if ( 0.75 >= Utility.RandomDouble() )
				{
					PackItem( RandomMagicItem.RandomMagicWeapon( (BaseWeapon)RandomMagicItem.RandomWeapon, 1, 5 ));
				}
				PackItem( RandomMagicItem.RandomMagicArmor( (BaseArmor)RandomMagicItem.RandomArmor, 1, 5 ));
			}
		}

		public override int TreasureMapLevel{ get{ return 3; } }

		public Cyclops( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}
	}
}